!*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2009-2010 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!*******************************************************************************
!  Content: DJACOBI RCI Example
!
!  The programm computes the Jacobi matrix of the function on the basis of RCI
!  using the central difference.
!*******************************************************************************

      PROGRAM JACOBI_MATRIX 
      IMPLICIT NONE 
C**
      INCLUDE 'mkl_rci.fi'
C**
      EXTERNAL EXTENDET_POWELL 
C**
C** N - Number of function variables 
C** M - Dimension of function value 
      INTEGER N, M, I 
      PARAMETER (N = 4)
      PARAMETER (M = 4)
C**
C** Jacobi matrix 
      DOUBLE PRECISION A (M,N)
C** Solution vector. contains values x for f(x) 
C** Temporary arrays f1 & f2 which contains f1 = f(x+eps) | f2 = f(x-eps) 
      DOUBLE PRECISION F1(M), F2(M), X(N)
C** Precisions for jacobi_matrix calculation 
      DOUBLE PRECISION EPS 
C**
C** Jacobi-matrix solver handle 
      INTEGER*8 HANDLE 
C** Controls of rci cycle 
      INTEGER SUCCESSFUL, RCI_REQUEST 
      INTEGER RESULT
C**
C** Set the x values 
C** X   = 10.D0
      DO I = 1, N
         X(I) = 10.0D0
      END DO
C**
      EPS = 1.D-6
      PRINT *, 'START TESTING ...'
C** Initialize solver (allocate memory, set initial values) 
      RESULT = DJACOBI_INIT (HANDLE, N, M, X, A, EPS)
      IF ( RESULT.NE.TR_SUCCESS ) THEN 
C** If function does not complete successfully then print error message 
         PRINT *, '#FAIL: ERROR IN DJACOBI_INIT' 
         CALL  MKL_FREEBUFFERS
         STOP 1
      ENDIF 
C** Set initial rci cycle variables 
      RCI_REQUEST = 0 
      SUCCESSFUL  = 0 
C** Rci cycle 
      DO WHILE (SUCCESSFUL.EQ.0) 
C** Call solver 
         IF (DJACOBI_SOLVE (HANDLE, F1, F2, RCI_REQUEST).NE.TR_SUCCESS)
     +	THEN 
C** If function does not complete successfully then print error message 
            PRINT *, '#FAIL: ERROR IN DJACOBI_SOLVE'
            CALL  MKL_FREEBUFFERS
            STOP 1
         ENDIF 
         IF (RCI_REQUEST.EQ.1) THEN 
C** Calculate the function value f1 = f(x+eps) 
            CALL EXTENDET_POWELL (M, N, X, F1)
         ELSE IF (RCI_REQUEST.EQ.2) THEN 
C** Calculate the function value f2 = f(x-eps) 
            CALL EXTENDET_POWELL (M, N, X, F2) 
         ELSE IF (RCI_REQUEST.EQ.0) THEN 
C** Exit rci cycle 
            SUCCESSFUL = 1 
         ENDIF
      ENDDO 
C** Free handle memory 
      IF (DJACOBI_DELETE (HANDLE).NE.TR_SUCCESS) THEN 
C** If function does not complete successfully then print error message 
         PRINT *, '#FAIL: ERROR IN DJACOBI_DELETE' 
         CALL  MKL_FREEBUFFERS
         STOP 1
      ENDIF 
      PRINT *, '#PASS'
      STOP 0
      END PROGRAM JACOBI_MATRIX 
C**
C** Routine for extendet powell function calculation 
C** M in: dimension of function value 
C** N in: number of function variables 
C** X in: vector for function calculation 
C** F out: function value f(x) 
C**
      SUBROUTINE EXTENDET_POWELL (M, N, X, F) 
      IMPLICIT NONE 
      INTEGER M, N 
      DOUBLE PRECISION X (*), F (*) 
      INTEGER I 
         DO I = 1, N/4 
            F (4*I-3) = X(4*I - 3) + 10.D0 * X(4*I - 2) 
            F (4*I-2) = DSQRT(5.D0) * (X(4*I-1) - X(4*I)) 
            F (4*I-1) = (X(4*I-2) - 2.D0*X(4*I-1))**2 
            F (4*I)   = DSQRT(10.D0)*(X(4*I-3) - X(4*I))**2 
         ENDDO 
      END SUBROUTINE EXTENDET_POWELL
