/*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2006-2010 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!******************************************************************************
!   Content:
!       Wrapper library building. Function fftw_mpi_create_plan.
!       This library allows to use MKL CDFT routines through MPI FFTW interface.
!******************************************************************************/
#include "common.h"

#ifdef MKL_SINGLE
    #define PREC DFTI_SINGLE
#else
    #define PREC DFTI_DOUBLE
#endif

fftw_mpi_plan fftw_mpi_create_plan(MPI_Comm comm,MKL_INT n,fftw_direction dir,int flags) {

	MKL_LONG err;
	fftw_mpi_plan p;
	if ((flags!=FFTW_ESTIMATE)&&(flags!=FFTW_MEASURE))
	    if ((!(flags & FFTW_SCRAMBLED_OUTPUT))&&(!(flags & FFTW_SCRAMBLED_INPUT))) goto error2;

	p=(fftw_mpi_plan)malloc(sizeof(fftw_mpi_plan_struct));
	p->dir=dir;
	err=DftiCreateDescriptorDM(MKL_Comm_c2f(comm),&p->h,PREC,DFTI_COMPLEX,1,(MKL_LONG)n);
	if (err!=DFTI_NO_ERROR) goto error;

	if ((flags & FFTW_SCRAMBLED_OUTPUT)||(flags & FFTW_SCRAMBLED_INPUT) )
	{
	    err = DftiSetValueDM(p->h,DFTI_ORDERING, DFTI_ORDERED);
	    if (err!=DFTI_NO_ERROR) goto error;
	}

	err=DftiCommitDescriptorDM(p->h);
	if (err!=DFTI_NO_ERROR) goto error;
	return p;
error:
	fprintf(stderr,"CDFT error %ld in fftw_mpi_create_plan wrapper\n",err);
	free(p);
	return NULL;
error2:
	fprintf(stderr,"CDFT error in fftw_mpi_create_plan wrapper: unknown flags\n");
	return NULL;
}
