/*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2006-2010 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!******************************************************************************
!   Content:
!       Wrapper library building. Functions rfftw2d_mpi_create_plan,
!	rfftw3d_mpi_create_plan, rfftwnd_mpi_create_plan.
!       This library allows to use MKL CDFT routines through MPI FFTW interface.
!******************************************************************************/

#include "common.h"

#ifdef MKL_SINGLE
	#define PREC DFTI_SINGLE
#else
	#define PREC DFTI_DOUBLE
#endif


rfftwnd_mpi_plan rfftw2d_mpi_create_plan(MPI_Comm comm, int nx, int ny, fftw_direction dir, int flags)
{
	MKL_LONG err;
	rfftwnd_mpi_plan p;
	MKL_LONG n[2];
	if ((flags!=FFTW_ESTIMATE)&&(flags!=FFTW_MEASURE))
		if ((!(flags & FFTW_SCRAMBLED_OUTPUT))&&(!(flags & FFTW_SCRAMBLED_INPUT))) goto error2;

	p=(rfftwnd_mpi_plan)malloc(sizeof(fftwnd_mpi_plan_struct));
	if (p==NULL)
	{
		fprintf(stderr,"CDFT error in rfftw2d_mpi_create_plan wrapper: memory can not be allocated\n");
		return NULL;
	}

	p->dir=dir;

	n[0]=nx; n[1]=ny;
	err=DftiCreateDescriptorDM(MKL_Comm_c2f(comm),&p->h,PREC,DFTI_REAL,2,n);
	if (err!=DFTI_NO_ERROR) goto error;

	if ((flags & FFTW_SCRAMBLED_OUTPUT)||(flags & FFTW_SCRAMBLED_INPUT) )
	{
		err = DftiSetValueDM(p->h,DFTI_ORDERING, DFTI_ORDERED);
		if (err!=DFTI_NO_ERROR)
		{
			DftiFreeDescriptorDM(&p->h);
			goto error;
		}
	}
	err=DftiCommitDescriptorDM(p->h);
	if (err!=DFTI_NO_ERROR) goto error;
	return p;

error:
	fprintf(stderr,"CDFT error %ld in rfftw2d_mpi_create_plan wrapper\n",err);
	free(p);
	return NULL;
error2:
	fprintf(stderr,"CDFT error  in rfftw2d_mpi_create_plan wrapper: unknown flags\n");
	return NULL;
}

void rfftw2d_f77_mpi_create_plan(rfftwnd_mpi_plan *plan, MPI_Comm *comm, int *nx, int *ny, fftw_direction *dir, int *flags)
{
	*plan = rfftw2d_mpi_create_plan(MKL_Comm_c2f(MPI_COMM_WORLD), *nx, *ny, *dir, *flags);
}

rfftwnd_mpi_plan rfftw3d_mpi_create_plan(MPI_Comm comm, int nx, int ny, int nz, fftw_direction dir, int flags)
{
	MKL_LONG err;
	rfftwnd_mpi_plan p;
	MKL_LONG n[3];
	if ((flags!=FFTW_ESTIMATE)&&(flags!=FFTW_MEASURE))
		if ((!(flags & FFTW_SCRAMBLED_OUTPUT))&&(!(flags & FFTW_SCRAMBLED_INPUT))) goto error2;

	p=(rfftwnd_mpi_plan)malloc(sizeof(fftwnd_mpi_plan_struct));
	if (p==NULL)
	{
		fprintf(stderr,"CDFT error in rfftw3d_mpi_create_plan wrapper: memory can not be allocated\n");
		return NULL;
	}

	p->dir=dir;

	n[0]=nx; n[1]=ny; n[2]=nz;
	err=DftiCreateDescriptorDM(MKL_Comm_c2f(comm),&p->h,PREC,DFTI_REAL,3,n);
	if (err!=DFTI_NO_ERROR) goto error;

	if ((flags & FFTW_SCRAMBLED_OUTPUT)||(flags & FFTW_SCRAMBLED_INPUT) )
	{
		err = DftiSetValueDM(p->h,DFTI_ORDERING, DFTI_ORDERED);
		if (err!=DFTI_NO_ERROR)
		{
			DftiFreeDescriptorDM(&p->h);
			goto error;
		}
	}

	err=DftiCommitDescriptorDM(p->h);
	if (err!=DFTI_NO_ERROR) goto error;

	return p;

error:
	fprintf(stderr,"CDFT error %ld in rfftw3d_mpi_create_plan wrapper\n",err);
	free(p);
	return NULL;
error2:
	fprintf(stderr,"CDFT error  in rfftw3d_mpi_create_plan wrapper: unknown flags\n");
	return NULL;
}

void rfftw3d_f77_mpi_create_plan(rfftwnd_mpi_plan *plan, MPI_Comm *comm, int *nx, int *ny, int *nz, fftw_direction *dir, int *flags)
{
	*plan = rfftw3d_mpi_create_plan(MKL_Comm_c2f(MPI_COMM_WORLD), *nx, *ny, *nz, *dir, *flags);
}

rfftwnd_mpi_plan rfftwnd_mpi_create_plan(MPI_Comm comm, int dim, int *nn, fftw_direction dir, int flags)
{
	MKL_LONG err;
	rfftwnd_mpi_plan p;
	MKL_LONG *n;
	int i;
	if ((flags!=FFTW_ESTIMATE)&&(flags!=FFTW_MEASURE))
		if ((!(flags & FFTW_SCRAMBLED_OUTPUT))&&(!(flags & FFTW_SCRAMBLED_INPUT))) goto error2;

	p=(rfftwnd_mpi_plan)malloc(sizeof(fftwnd_mpi_plan_struct));
	if (p==NULL)
	{
		fprintf(stderr,"CDFT error in rfftwnd_mpi_create_plan wrapper: memory can not be allocated\n");
		return NULL;
	}

	p->dir=dir;

	n=(MKL_LONG*)malloc(dim*sizeof(MKL_LONG));
	if (n==NULL)
	{
		free(p);
		fprintf(stderr,"CDFT error in rfftwnd_mpi_create_plan wrapper: memory can not be allocated\n");
		return NULL;
	}
	for( i=0; i<dim; i++) n[i]=nn[i];

	if (dim==1)
		err=DftiCreateDescriptorDM(MKL_Comm_c2f(comm),&p->h,PREC,DFTI_REAL,1,n[0]);
	else
		err=DftiCreateDescriptorDM(MKL_Comm_c2f(comm),&p->h,PREC,DFTI_REAL,dim,n);
	free(n);
	if (err!=DFTI_NO_ERROR) goto error;

	if ((flags & FFTW_SCRAMBLED_OUTPUT)||(flags & FFTW_SCRAMBLED_INPUT) )
	{
		err = DftiSetValueDM(p->h,DFTI_ORDERING, DFTI_ORDERED);
		if (err!=DFTI_NO_ERROR)
		{
			DftiFreeDescriptorDM(&p->h);
			goto error;
		}
	}

	err=DftiCommitDescriptorDM(p->h);
	if (err!=DFTI_NO_ERROR) goto error;

	return p;

error:
	fprintf(stderr,"CDFT error %ld in rfftwnd_mpi_create_plan wrapper\n",err);
	free(p);
	return NULL;

error2:
	fprintf(stderr,"CDFT error in rfftwnd_mpi_create_plan wrapper: unknown flags\n");
	return NULL;
}

void rfftwnd_f77_mpi_create_plan(rfftwnd_mpi_plan *plan, MPI_Comm *comm, int *dim, int *nn, fftw_direction *dir, int *flags)
{
	*plan = rfftwnd_mpi_create_plan(MKL_Comm_c2f(MPI_COMM_WORLD), *dim, nn, *dir, *flags);
}

