/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * fftw_plan_dft_c2r - FFTW3 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw3_mkl.h"

fftw_plan
fftw_plan_dft_c2r(int rank, const int *n, fftw_complex *in, double *out,
                  unsigned flags)
{
    fftw_iodim64 dims64[MKL_MAXRANK];
    int i, inplace;

    if (rank > MKL_MAXRANK)
        return NULL;

    for (i = 0; i < rank; ++i)
        dims64[i].n = n[i];

    /* compute strides, different for inplace and out-of-place */
    inplace = ((void *)in == (void *)out);
    if (rank > 0)
    {
        dims64[rank - 1].is = 1;
        dims64[rank - 1].os = 1;
    }
    if (rank > 1)
    {
        dims64[rank - 2].is = dims64[rank - 1].n / 2 + 1;
        dims64[rank - 2].os =
            (inplace ? dims64[rank - 2].is * 2 : dims64[rank - 1].n);
    }
    for (i = rank - 2; i > 0; --i)
    {
        dims64[i - 1].is = dims64[i].is * dims64[i].n;
        dims64[i - 1].os = dims64[i].os * dims64[i].n;
    }

    return fftw_plan_guru64_dft_c2r(rank, dims64, 0, NULL, in, out, flags);
}
