/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * dfftw_plan_dft - FFTW3 Fortran 77 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw3_mkl_f77.h"

void
dfftw_plan_dft(PLAN *p, INTEGER *rank, INTEGER *n, COMPLEX16 *in,
               COMPLEX16 *out, INTEGER *sign, INTEGER *flags)
{
    fftw_iodim64 dims64[MKL_MAXRANK];
    int i;

    if (*rank > MKL_MAXRANK)
    {
        *(fftw_plan *)p = NULL;
        return;
    }

    /* Reverse dimensions for column-major layout */
    for (i = 0; i < *rank; i++)
    {
        int j = *rank - i - 1;
        dims64[j].n = n[i];
        if (i == 0)
        {
            dims64[j].is = 1;
            dims64[j].os = 1;
        }
        else
        {
            dims64[j].is = dims64[j + 1].is * dims64[j + 1].n;
            dims64[j].os = dims64[j + 1].os * dims64[j + 1].n;
        }
    }

    *(fftw_plan *)p =
        fftw_plan_guru64_dft(*rank, dims64, 0, NULL, in, out, *sign, *flags);
}
