/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * fftw_version etc - FFTW3 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw3_mkl.h"

const char fftw_version[] = "FFTW 3.2 wrappers to MKL";
const char fftw_cc[] = "";
const char fftw_codelet_optim[] = "";

/* Supplementary definitions for the wrappers */

static fftw_mkl_plan
new_plan(void)
{
    fftw_mkl_plan p = (fftw_mkl_plan)fftw_malloc(sizeof(*p));

    if (p)
    {
        p->execute = NULL;
        p->desc = NULL;
        p->io[0] = p->io[1] = p->io[2] = p->io[3] = 0;
        p->ipar = NULL;
        p->dpar = NULL;
        p->spar = NULL;
    }
    return p;
}

static void
delete_plan(fftw_mkl_plan p)
{
    if (p)
    {
        if (p->spar)
            fftw_free(p->spar);
        if (p->dpar)
            fftw_free(p->dpar);
        if (p->ipar)
            fftw_free(p->ipar);
        if (p->desc)
            DftiFreeDescriptor(&p->desc);
        fftw_free(p);
    }
}

#ifndef MKL_FFTW_MALLOC_ALIGNMENT
#define MKL_FFTW_MALLOC_ALIGNMENT (16)
#endif

fftw3_mkl_s fftw3_mkl = {
    0,  /* verbose */
    1,  /* nthreads */
    0.0,        /* timelimit */
    1,  /* number_of_user_threads */
    new_plan,
    delete_plan,
    MKL_FFTW_MALLOC_ALIGNMENT /* default_alignment */
};
